@@ -1,7 +1,7 @@
 /*
  * zmath - declarations for extended precision integer arithmetic
  *
- * Copyright (C) 1999-2007,2014  David I. Bell
+ * Copyright (C) 1999-2007,2014,2021  David I. Bell
  *
  * Calc is open software; you can redistribute it and/or modify it under
  * the terms of the version 2.1 of the GNU Lesser General Public License
@@ -26,7 +26,7 @@
 /*
  * Data structure declarations for extended precision integer arithmetic.
  * The assumption made is that a long is 32 bits and shorts are 16 bits,
- * and longs must be addressible on word boundaries.
+ * and longs must be addressable on word boundaries.
  */
 
 
@@ -121,6 +121,20 @@ typedef SB32 SFULL;			/* signed FULL */
 #define BASE1	(BASE - (FULL)1)		/* one less than base */
 #define BASEDIG ((BASEB/16)*5)			/* number of digits in base */
 #define FULL_BITS (2*BASEB)			/* bits in a FULL */
+#define HALF_LEN (sizeof(HALF))			/* length of HALF in bites */
+#define FULL_LEN (sizeof(FULL))			/* length of FULL in bites */
+
+/*
+ * ROUNDUP(value, mult) - round up value to the next multiple of mult
+ *
+ * NOTE: value and mult musty be of an integer type.
+ *
+ * NOTE: mult must != 0
+ *
+ * NOTE: If value is a multiple of mult, then ROUNDUP(value, mult)
+ *	 will just return value.
+ */
+#define ROUNDUP(value, mult) ( ( ((value)+(mult)-1) / (mult) ) * (mult) )
 
 #define TOPHALF ((FULL)1 << (BASEB-1))		/* highest bit in a HALF */
 #define MAXHALF (TOPHALF - (FULL)1)		/* largest SHALF value */
@@ -140,11 +154,11 @@ typedef SB32 SFULL;			/* signed FULL */
  */
 typedef USB32 QCKHASH;			/* 32 bit hash value */
 #if defined(HAVE_B64) && LONG_BITS == 32
-typedef HALF PRINT;			/* cast for zio printing fucctions */
+typedef HALF PRINT;			/* cast for zio printing functions */
 #define SWAP_B16_IN_PRINT(dest, src)	SWAP_B16_IN_HALF(dest, src)
 #define SWAP_B8_IN_PRINT(dest, src)	SWAP_B8_IN_HALF(dest, src)
 #else
-typedef FULL PRINT;			/* cast for zio printing fucctions */
+typedef FULL PRINT;			/* cast for zio printing functions */
 #define SWAP_B16_IN_PRINT(dest, src)	SWAP_B16_IN_FULL(dest, src)
 #define SWAP_B8_IN_PRINT(dest, src)	SWAP_B8_IN_FULL(dest, src)
 #endif
@@ -474,7 +488,7 @@ E_FUNC void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 
 /*
  * Some algorithms testing for values of a certain length.  Macros such as
- * zistiny() do this well.  In other cases algorthms require tests for values
+ * zistiny() do this well.  In other cases algorithms require tests for values
  * in comparison to a given power of 2.	 In the later case, zistiny() compares
  * against a different power of 2 on a 64 bit machine.	The macros below
  * provide a tests against powers of 2 that are independent of the work size.
@@ -579,10 +593,17 @@ E_FUNC void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 #define MODE_HEX	5
 #define MODE_OCTAL	6
 #define MODE_BINARY	7
-#define MODE_MAX	7
+#define MODE_REAL_AUTO	8
+#define MODE_ENG    9
+#define MODE_MAX	9
 #define MODE2_OFF	(MODE_MAX+1)
 
+/* XXX - perhaps we need the MODE_REAL_AUTO vs MODE_REAL as a config mode? */
+#if 0 	/* XXX - can we safely set MODE_INITIAL to MODE_REAL_AUTO ?? */
+#define MODE_INITIAL	MODE_REAL_AUTO
+#else
 #define MODE_INITIAL	MODE_REAL
+#endif
 #define MODE2_INITIAL	MODE2_OFF
 
 
@@ -627,6 +648,7 @@ EXTERN HALF _tenval_[], _elevenval_[], _twelveval_[], _thirteenval_[];
 EXTERN HALF _fourteenval_[], _fifteenval_[];
 EXTERN HALF _sqbaseval_[];
 EXTERN HALF _fourthbaseval_[];
+EXTERN HALF _threesixtyval_[], _fourhundredval_[], _twentyfourval_[];
 
 EXTERN ZVALUE zconst[];		/* ZVALUE integers from 0 thru 15 */
 
@@ -639,7 +661,7 @@ EXTERN BOOL _math_abort_;	/* nonzero to abort calculations */
 EXTERN ZVALUE _tenpowers_[];	/* table of 10^2^n */
 
 /*
- * Bit fiddeling functions and types
+ * Bit fiddling functions and types
  */
 EXTERN HALF bitmask[];		/* bit rotation, norm 0 */
 EXTERN HALF lowhalf[];		/* bit masks from low end of HALF */
